#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

/* move servo at the specified degree */
int move_servo(int degree)
{
	/* 112 - moves at 0 degrees */
	/* 550 - moves at 180 degrees */
	/* found the ecuation of a line passing through two points */
	return (((73 * degree) / 30) + 112);
}

#define F_CPU 16000000
#define AREF 5

#ifdef ADC_USE_IRQ
#define ADC_INIT_CHANNEL 0

int ADC_value[8];
ISR(TIMER1_COMPA_vect)
{
    // start an ADC conversion
    ADCSRA |= (1 << ADSC);
}

ISR(ADC_vect)
{
    // * store new result
    // * change channel
    // * start a new conversion if there are any channels left
    static uint8_t channel = ADC_INIT_CHANNEL;

    ADC_value[channel] = ADC;

    channel = (channel + 1) % 8;
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    if (channel != ADC_INIT_CHANNEL)
        ADCSRA |= (1 << ADSC);
}

#endif

void ADC_init(void)
{
#ifndef ADC_USE_IRQ
	// enable ADC with:
	// *reference AVCC with external capacitor at AREF pin
	// *without left adjust of conversion result
	// *no auto-trigger
	// *no interrupt
	// *prescaler at 128
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
#else
	// enable ADC with interrupt
	ADMUX = (1 << REFS0) | (1 << ADC_INIT_CHANNEL);
	ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADIE) | (7 << ADPS0);
#endif
}

int ADC_get(uint8_t channel)
{
#ifndef ADC_USE_IRQ
	// start ADC conversion on "channel"
	// wait for completion
	// return the result
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
#else
	return ADC_value[channel];
#endif

    (void)channel;
    return 0;
}

/*
    Initializes the timer used for starting ADC conversions.
*/
void Timer_init()
{
    // initialize timer to trigger COMPA every 100ms
    TCCR1B = (5 << CS10) | (1 << WGM12);
    TIMSK1 |= (1 << OCIE1A);
    OCR1A = 1561;
}

int main()
{
	int lum0, lum1, tolerance, diff0, diff1;
	ADC_init();

	/* non intverted PWM */
	TCCR1A |= (1<<COM1A1) | (1<<COM1B1) | (1<<WGM11);

	/* set prescaler 64, mode 14 (FAST PWM) */
	TCCR1B |= (1<<WGM13) | (1<<WGM12) | (1<<CS11) | (1<<CS10);

	 /* set frequency fPWM=50Hz */
	ICR1=1999;

	/* PWM pin set as output */
	DDRD |= (1<<PD5);

	tolerance = 5;

	/* servo goes to the initial position */
	OCR1A = move_servo(90);
	_delay_ms(2000);

	while (1) {

		/* light intensity from sensors */
		lum0 = ADC_get(0);
		lum1 = ADC_get(1);

		/* determine the direction of the most powerful light */
		diff0 = abs(lum0 - lum1);
		diff1 = abs(lum1 - lum0);

		/* if difference is too little, do not move the servo */
		if((diff0 <= tolerance) || (diff1 <= tolerance)) {
			continue;
		}
		else {

			/* move servo towards 0 degrees */
			if (lum0 > lum1 && OCR1A > 112) {
				OCR1A--;
			}

			/* move servo towards 180  degrees */
			if (lum1 > lum0 && OCR1A < 550) {
				OCR1A++;
			}
	  	}
		_delay_ms(100);
	}

	return 0;
	  
}
